@@ -128,7 +128,7 @@ set_dss_ready (MbimDevice *device,
     GError *error = NULL;
 
     response = mbim_device_command_finish (device, res, &error);
-    if (!response || !mbim_message_command_done_get_result (response, &error)) {
+    if (!response || !mbim_message_response_get_result (response, MBIM_MESSAGE_TYPE_COMMAND_DONE, &error)) {
         g_printerr ("error: operation failed: %s\n", error->message);
         g_error_free (error);
         shutdown (FALSE);
